"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getCacheInvalidations = exports.setSimpleAuthBehavior = exports.identifyingTag = exports.invalidateCloudfrontCache = exports.createCloudFrontDistribution = exports.tagCloudFrontDistribution = exports.findDeployedCloudfrontDistribution = void 0;

var _awsServices = require("./aws-services");

var _awsHelper = require("./aws-helper");

var _logger = require("./logger");

var _lambda = require("./lambda");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const findDeployedCloudfrontDistribution = async domainName => {
  const distributions = await (0, _awsHelper.getAll)(async (nextMarker, page) => {
    _logger.logger.info(`[CloudFront] 🔍 searching cloudfront distribution (page ${page})...`);

    const {
      DistributionList
    } = await _awsServices.cloudfront.listDistributions({
      Marker: nextMarker
    }).promise();

    if (!DistributionList) {
      return {
        items: [],
        nextMarker: undefined
      };
    }

    return {
      items: DistributionList.Items || [],
      nextMarker: DistributionList.NextMarker
    };
  });
  const distribution = distributions.find(_distribution => Boolean(_distribution.Aliases.Items && _distribution.Aliases.Items.includes(domainName)));

  if (!distribution) {
    _logger.logger.info(`[CloudFront] 😬 No matching distribution`);

    return null;
  }

  const {
    Tags
  } = await _awsServices.cloudfront.listTagsForResource({
    Resource: distribution.ARN
  }).promise();

  if (!Tags || !Tags.Items || !Tags.Items.find(tag => tag.Key === identifyingTag.Key && tag.Value === identifyingTag.Value)) {
    throw new Error(`CloudFront distribution ${distribution.Id} has no tag ${identifyingTag.Key}:${identifyingTag.Value}`);
  }

  _logger.logger.info(`[CloudFront] 👍 Distribution found: ${distribution.Id}`);

  if (["InProgress", "In Progress"].includes(distribution.Status)) {
    _logger.logger.info(`[CloudFront] ⏱ Waiting for distribution to be deployed. This step might takes up to 25 minutes...`);

    await _awsServices.cloudfront.waitFor("distributionDeployed", {
      Id: distribution.Id
    }).promise();
  }

  return distribution;
};

exports.findDeployedCloudfrontDistribution = findDeployedCloudfrontDistribution;

const tagCloudFrontDistribution = async distribution => {
  _logger.logger.info(`[CloudFront] ✏️ Tagging "${distribution.Id}" bucket with "${identifyingTag.Key}:${identifyingTag.Value}"...`);

  await _awsServices.cloudfront.tagResource({
    Resource: distribution.ARN,
    Tags: {
      Items: [identifyingTag]
    }
  }).promise();
};

exports.tagCloudFrontDistribution = tagCloudFrontDistribution;

const createCloudFrontDistribution = async (domainName, sslCertificateARN) => {
  _logger.logger.info(`[CloudFront] ✏️ Creating Cloudfront distribution with origin "${getS3DomainName(domainName)}"...`);

  const {
    Distribution
  } = await _awsServices.cloudfront.createDistribution({
    DistributionConfig: getDistributionConfig(domainName, sslCertificateARN)
  }).promise();

  if (!Distribution) {
    throw new Error("[CloudFront] Could not create distribution");
  }

  await tagCloudFrontDistribution(Distribution);

  _logger.logger.info(`[CloudFront] ⏱ Waiting for distribution to be available. This step might takes up to 25 minutes...`);

  await _awsServices.cloudfront.waitFor("distributionDeployed", {
    Id: Distribution.Id
  }).promise();
  return Distribution;
};

exports.createCloudFrontDistribution = createCloudFrontDistribution;

const getDistributionConfig = (domainName, sslCertificateARN) => ({
  CallerReference: Date.now().toString(),
  Aliases: {
    Quantity: 1,
    Items: [domainName]
  },
  Origins: {
    Quantity: 1,
    Items: [{
      Id: getOriginId(domainName),
      DomainName: getS3DomainName(domainName),
      CustomOriginConfig: {
        HTTPPort: 80,
        HTTPSPort: 443,
        OriginProtocolPolicy: "http-only",
        OriginSslProtocols: {
          Quantity: 1,
          Items: ["TLSv1"]
        },
        OriginReadTimeout: 30,
        OriginKeepaliveTimeout: 5
      },
      CustomHeaders: {
        Quantity: 0,
        Items: []
      },
      OriginPath: ""
    }]
  },
  Enabled: true,
  Comment: "",
  PriceClass: "PriceClass_All",
  Logging: {
    Enabled: false,
    IncludeCookies: false,
    Bucket: "",
    Prefix: ""
  },
  CacheBehaviors: {
    Quantity: 0
  },
  CustomErrorResponses: {
    Quantity: 0
  },
  Restrictions: {
    GeoRestriction: {
      RestrictionType: "none",
      Quantity: 0
    }
  },
  DefaultRootObject: "index.html",
  WebACLId: "",
  HttpVersion: "http2",
  DefaultCacheBehavior: {
    ViewerProtocolPolicy: "redirect-to-https",
    TargetOriginId: getOriginId(domainName),
    ForwardedValues: {
      QueryString: false,
      Cookies: {
        Forward: "none"
      },
      Headers: {
        Quantity: 0,
        Items: []
      },
      QueryStringCacheKeys: {
        Quantity: 0,
        Items: []
      }
    },
    AllowedMethods: {
      Quantity: 2,
      Items: ["HEAD", "GET"],
      CachedMethods: {
        Quantity: 2,
        Items: ["HEAD", "GET"]
      }
    },
    TrustedSigners: {
      Enabled: false,
      Quantity: 0
    },
    MinTTL: 0,
    DefaultTTL: 86400,
    MaxTTL: 31536000,
    FieldLevelEncryptionId: "",
    LambdaFunctionAssociations: {
      Quantity: 0,
      Items: []
    },
    SmoothStreaming: false,
    Compress: true // this is required to deliver gzip data

  },
  ViewerCertificate: {
    ACMCertificateArn: sslCertificateARN,
    SSLSupportMethod: "sni-only",
    MinimumProtocolVersion: "TLSv1.1_2016",
    CertificateSource: "acm"
  }
});

const getS3DomainName = domainName => `${domainName}.${_awsServices.websiteEndpoint[_awsServices.bucketRegion]}`;

const getOriginId = domainName => `S3-Website-${getS3DomainName(domainName)}`;

const invalidateCloudfrontCache = async (distributionId, paths, wait = false) => {
  _logger.logger.info("[CloudFront] ✏️ Creating invalidation...");

  const {
    Invalidation
  } = await _awsServices.cloudfront.createInvalidation({
    DistributionId: distributionId,
    InvalidationBatch: {
      CallerReference: Date.now().toString(),
      Paths: {
        Quantity: paths.split(",").length,
        Items: paths.split(",").map(path => path.trim())
      }
    }
  }).promise();

  if (!Invalidation) {
    return;
  }

  if (wait) {
    _logger.logger.info("[CloudFront] ⏱ Waiting for invalidation to be completed (can take up to 10 minutes)...");

    await _awsServices.cloudfront.waitFor("invalidationCompleted", {
      DistributionId: distributionId,
      Id: Invalidation.Id
    }).promise();
  }
};

exports.invalidateCloudfrontCache = invalidateCloudfrontCache;
const identifyingTag = {
  Key: "managed-by-aws-spa",
  Value: "v1"
};
exports.identifyingTag = identifyingTag;

const setSimpleAuthBehavior = async (distributionId, lambdaFunctionARN) => {
  const {
    DistributionConfig,
    ETag
  } = await _awsServices.cloudfront.getDistributionConfig({
    Id: distributionId
  }).promise();
  const lambdaConfigs = DistributionConfig.DefaultCacheBehavior.LambdaFunctionAssociations.Items;

  if (lambdaFunctionARN === null) {
    _logger.logger.info(`[CloudFront] 📚 No basic auth configured. Checking if there is a basic auth to remove...`);

    const updatedLambdaFunctions = lambdaConfigs.filter(config => !config.LambdaFunctionARN.includes(_lambda.lambdaPrefix));

    if (updatedLambdaFunctions.length !== lambdaConfigs.length) {
      _logger.logger.info(`[CloudFront] 🗑 Removing lambda function association handling basic auth...`);

      await updateLambdaFunctionAssociations(distributionId, DistributionConfig, updatedLambdaFunctions, ETag);

      _logger.logger.info(`[CloudFront] 👍 Lambda function association removed`);
    } else {
      _logger.logger.info(`[CloudFront] 👍 No basic auth setup`);
    }

    return;
  }

  _logger.logger.info(`[CloudFront] 📚 Checking if basic auth is already setup...`);

  console.log(lambdaConfigs, lambdaFunctionARN);

  if (lambdaConfigs.find(config => config.LambdaFunctionARN === lambdaFunctionARN)) {
    _logger.logger.info(`[CloudFront] 👍 Basic auth already setup`);

    return;
  }

  _logger.logger.info(`[CloudFront] ✏️ Adding simple auth behavior (and replacing "viewer-request" lambda if any)...`);

  await updateLambdaFunctionAssociations(distributionId, DistributionConfig, [...lambdaConfigs.filter(config => config.EventType !== "viewer-request"), {
    LambdaFunctionARN: lambdaFunctionARN,
    EventType: "viewer-request",
    IncludeBody: false
  }], ETag);
};

exports.setSimpleAuthBehavior = setSimpleAuthBehavior;

const getCacheInvalidations = (cacheInvalidations, subFolder) => cacheInvalidations.split(",").map(string => string.trim().replace(/^\//, "")).map(string => subFolder ? `/${subFolder}/${string}` : `/${string}`).join(",");

exports.getCacheInvalidations = getCacheInvalidations;

const updateLambdaFunctionAssociations = async (distributionId, DistributionConfig, lambdaConfigs, ETag) => {
  await _awsServices.cloudfront.updateDistribution({
    Id: distributionId,
    IfMatch: ETag,
    DistributionConfig: _objectSpread({}, DistributionConfig, {
      DefaultCacheBehavior: _objectSpread({}, DistributionConfig.DefaultCacheBehavior, {
        LambdaFunctionAssociations: {
          Quantity: lambdaConfigs.length,
          Items: lambdaConfigs
        }
      })
    })
  }).promise();
};