"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deploy = void 0;

var _fs = require("fs");

var _s = require("./s3");

var _acm = require("./acm");

var _cloudfront = require("./cloudfront");

var _route = require("./route53");

var _logger = require("./logger");

var _lambda = require("./lambda");

var _prompt = require("./prompt");

const deploy = async (url, folder, wait, cacheInvalidations, cacheBustedPrefix, credentials, noPrompt) => {
  await (0, _prompt.predeployPrompt)(Boolean(process.env.CI), noPrompt);
  const [domainName, s3Folder] = url.split("/");

  _logger.logger.info(`✨ Deploying "${folder}" on "${domainName}" with path "${s3Folder || "/"}"...`);

  if (!(0, _fs.existsSync)(folder)) {
    throw new Error(`folder "${folder}" not found`);
  }

  if (!(0, _fs.existsSync)(`${folder}/index.html`)) {
    throw new Error(`"index.html" not found in "${folder}" folder`);
  }

  if (await (0, _s.doesS3BucketExists)(domainName)) {
    await (0, _s.confirmBucketManagement)(domainName);
  } else {
    await (0, _s.createBucket)(domainName); // without this timeout `setBucketPolicy` fails with error
    // "The specified bucket does not exist"

    await new Promise(resolve => setTimeout(resolve, 2000));
  }

  await (0, _s.tagBucket)(domainName);
  await (0, _s.setBucketWebsite)(domainName);
  await (0, _s.setBucketPolicy)(domainName);
  let hostedZone = await (0, _route.findHostedZone)(domainName);

  if (!hostedZone) {
    hostedZone = await (0, _route.createHostedZone)(domainName);
  }

  let certificateArn = await (0, _acm.getCertificateARN)(domainName);

  if (!certificateArn) {
    certificateArn = await (0, _acm.createCertificate)(domainName, hostedZone.Id);
  }

  let distribution = await (0, _cloudfront.findDeployedCloudfrontDistribution)(domainName);

  if (!distribution) {
    distribution = await (0, _cloudfront.createCloudFrontDistribution)(domainName, certificateArn);
  }

  if (credentials) {
    const simpleAuthLambdaARN = await (0, _lambda.deploySimpleAuthLambda)(domainName, credentials);
    await (0, _cloudfront.setSimpleAuthBehavior)(distribution.Id, simpleAuthLambdaARN);
  } else {
    await (0, _cloudfront.setSimpleAuthBehavior)(distribution.Id, null);
  }

  if (await (0, _route.needsUpdateRecord)(hostedZone.Id, domainName, distribution.DomainName)) {
    await (0, _route.updateRecord)(hostedZone.Id, domainName, distribution.DomainName);
  }

  await (0, _s.syncToS3)(folder, domainName, cacheBustedPrefix, s3Folder);
  await (0, _cloudfront.invalidateCloudfrontCache)(distribution.Id, (0, _cloudfront.getCacheInvalidations)(cacheInvalidations, s3Folder), wait);
};

exports.deploy = deploy;