"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDescription = exports.deploySimpleAuthLambda = exports.lambdaPrefix = void 0;

var _admZip = _interopRequireDefault(require("adm-zip"));

var _logger = require("./logger");

var _awsServices = require("./aws-services");

var _iam = require("./iam");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const lambdaPrefix = `aws-spa-basic-auth-`;
exports.lambdaPrefix = lambdaPrefix;

const deploySimpleAuthLambda = async (domainName, credentials) => {
  const name = `${lambdaPrefix}${domainName.replace(/\./g, "-")}`;

  if (!(await doesFunctionExists(name))) {
    const roleARN = await (0, _iam.getRoleARNForBasicLambdaExectution)(name);

    _logger.logger.info(`[Lambda] ✏️ Creating lambda function...`);

    await _awsServices.lambda.createFunction({
      Code: {
        ZipFile: getZippedCode(credentials)
      },
      FunctionName: name,
      Handler: "simple-auth.handler",
      Role: roleARN,
      Runtime: "nodejs14.x",
      Description: getDescription(credentials),
      Publish: true
    }).promise();

    _logger.logger.info(`[Lambda] 👍 lambda created`);
  }

  _logger.logger.info(`[Lambda] 🔍 Checking if credentials changed...`);

  const {
    FunctionArn,
    Description,
    Version
  } = await _awsServices.lambda.getFunctionConfiguration({
    FunctionName: name
  }).promise();

  if (Description && Description === getDescription(credentials)) {
    _logger.logger.info(`[Lambda] 👍 credentials didn't changed. Everything is fine.`);

    return `${FunctionArn}:${Version === "$LATEST" ? "1" : Version}`;
  }

  _logger.logger.info(`[Lambda] ✏️ Credentials changed. Updating code...`);

  const {
    Version: newVersion
  } = await _awsServices.lambda.updateFunctionCode({
    FunctionName: name,
    ZipFile: getZippedCode(credentials),
    Publish: true
  }).promise();
  await _awsServices.lambda.updateFunctionConfiguration({
    FunctionName: name,
    Description: getDescription(credentials)
  }).promise();

  _logger.logger.info(`[Lambda] 👍 Code updated`);

  return `${FunctionArn}:${newVersion}`;
};

exports.deploySimpleAuthLambda = deploySimpleAuthLambda;

const doesFunctionExists = async functionName => {
  try {
    _logger.logger.info(`[Lambda] 🔍 Searching lambda function "${functionName}"...`);

    await _awsServices.lambda.getFunction({
      FunctionName: functionName
    }).promise();

    _logger.logger.info(`[Lambda] 👍 lambda function found`);

    return true;
  } catch (error) {
    if (error.statusCode === 404) {
      _logger.logger.info(`[Lambda] 😬 No lambda found`);

      return false;
    }

    throw error;
  }
};

const getDescription = credentials => `Deployed by aws-spa to handle simple auth [credentials=${credentials}]`;

exports.getDescription = getDescription;

const getZippedCode = credentials => {
  const zip = new _admZip.default();
  zip.addFile("simple-auth.js", Buffer.from(getLambdaCode(credentials)));
  return zip.toBuffer();
}; // lambda@edge does not allow to use env variables :-/


const getLambdaCode = credentials => `
exports.handler = (event, context, callback) => {
  const request = event.Records[0].cf.request;
  const headers = request.headers;

  const authString =
    "Basic " + new Buffer("${credentials}").toString("base64");

  if (
    typeof headers.authorization == "undefined" ||
    headers.authorization[0].value != authString
  ) {
    const body = "Unauthorized";
    const response = {
      status: "401",
      statusDescription: "Unauthorized",
      body: body,
      headers: {
        "www-authenticate": [{ key: "WWW-Authenticate", value: "Basic" }]
      }
    };
    callback(null, response);
  }

  callback(null, request);
};
`;