"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.syncToS3 = exports.identifyingTag = exports.setBucketPolicy = exports.setBucketWebsite = exports.tagBucket = exports.confirmBucketManagement = exports.createBucket = exports.doesS3BucketExists = void 0;

var _fs = require("fs");

var _mimeTypes = require("mime-types");

var _inquirer = require("inquirer");

var _awsServices = require("./aws-services");

var _fsHelper = require("./fs-helper");

var _logger = require("./logger");

const doesS3BucketExists = async bucketName => {
  try {
    _logger.logger.info(`[S3] 🔍 Looking for bucket "${bucketName}"...`);

    await _awsServices.s3.headBucket({
      Bucket: bucketName
    }).promise();
  } catch (error) {
    if (error.statusCode === 404) {
      _logger.logger.info(`[S3] 😬 Bucket "${bucketName}" not found...`);

      return false;
    }

    throw error;
  }

  _logger.logger.info(`[S3] 🔍 Bucket "${bucketName}" found`);

  return true;
};

exports.doesS3BucketExists = doesS3BucketExists;

const createBucket = async bucketName => {
  _logger.logger.info(`[S3] ✏️ Creating "${bucketName}" bucket...`);

  try {
    await _awsServices.s3.createBucket({
      Bucket: bucketName
    }).promise();
  } catch (error) {
    if (error.statusCode === 409) {
      throw new Error("[S3] It seems that a bucket already exists but in an unsupported region... You should delete it first.");
    }

    throw error;
  }
};

exports.createBucket = createBucket;

const confirmBucketManagement = async bucketName => {
  _logger.logger.info(`[S3] 🔍 Checking that tag "${identifyingTag.Key}:${identifyingTag.Value}" exists on bucket "${bucketName}"...`);

  try {
    const {
      TagSet
    } = await _awsServices.s3.getBucketTagging({
      Bucket: bucketName
    }).promise();
    const tag = TagSet.find(_tag => _tag.Key === identifyingTag.Key && _tag.Value === identifyingTag.Value);

    if (tag) {
      _logger.logger.info(`[S3] 👍 Tag "${identifyingTag.Key}:${identifyingTag.Value}" found`);

      return true;
    }
  } catch (error) {
    if (error.statusCode !== 404) {
      throw error;
    }
  }

  const {
    continueUpdate
  } = await (0, _inquirer.prompt)([{
    type: "confirm",
    name: "continueUpdate",
    message: `[S3] Bucket "${bucketName}" is not yet managed by aws-spa. Would you like it to be modified (public access & website config) & managed by aws-spa?`,
    default: false
  }]);

  if (continueUpdate) {
    return true;
  }

  throw new Error("You can use another domain name or delete the S3 bucket...");
};

exports.confirmBucketManagement = confirmBucketManagement;

const tagBucket = async bucketName => {
  _logger.logger.info(`[S3] ✏️ Tagging "${bucketName}" bucket with "${identifyingTag.Key}:${identifyingTag.Value}"...`);

  await _awsServices.s3.putBucketTagging({
    Bucket: bucketName,
    Tagging: {
      TagSet: [identifyingTag]
    }
  }).promise();
};

exports.tagBucket = tagBucket;

const setBucketWebsite = bucketName => {
  _logger.logger.info(`[S3] ✏️ Set bucket website with IndexDocument: "index.html" & ErrorDocument: "index.html" to "${bucketName}"...`);

  return _awsServices.s3.putBucketWebsite({
    Bucket: bucketName,
    WebsiteConfiguration: {
      ErrorDocument: {
        Key: "index.html"
      },
      IndexDocument: {
        Suffix: "index.html"
      }
    }
  }).promise();
};

exports.setBucketWebsite = setBucketWebsite;

const setBucketPolicy = bucketName => {
  _logger.logger.info(`[S3] ✏️ Allow public read to "${bucketName}"...`);

  return _awsServices.s3.putBucketPolicy({
    Bucket: bucketName,
    Policy: JSON.stringify({
      Statement: [{
        Sid: "AllowPublicRead",
        Effect: "Allow",
        Principal: {
          AWS: "*"
        },
        Action: "s3:GetObject",
        Resource: `arn:aws:s3:::${bucketName}/*`
      }]
    })
  }).promise();
};

exports.setBucketPolicy = setBucketPolicy;
const identifyingTag = {
  Key: "managed-by-aws-spa",
  Value: "v1"
};
exports.identifyingTag = identifyingTag;

const syncToS3 = function (folder, bucketName, cacheBustedPrefix, subfolder) {
  _logger.logger.info(`[S3] ✏️ Uploading "${folder}" folder on "${bucketName}"...`);

  const filesToUpload = (0, _fsHelper.readRecursively)(folder);
  return Promise.all(filesToUpload.map(file => {
    const filenameParts = file.split(".");
    const key = file.replace(`${folder}/`, "");
    const prefix = subfolder ? `${subfolder}/` : "";
    return _awsServices.s3.putObject({
      Bucket: bucketName,
      Key: `${prefix}${key}`,
      Body: (0, _fs.createReadStream)(file),
      CacheControl: getCacheControl(key, cacheBustedPrefix),
      ContentType: (0, _mimeTypes.lookup)(filenameParts[filenameParts.length - 1]) || "application/octet-stream"
    }).promise();
  }));
};

exports.syncToS3 = syncToS3;

const getCacheControl = (filename, cacheBustedPrefix) => {
  if (filename === "index.html") {
    // This will allow CloudFront to store the file on the edge location,
    // but it will force it to revalidate it with the origin with each request.
    // If the file hasn't changed, CloudFront will not need to transfer the
    // file's entire content from the origin.
    return "public, must-revalidate, proxy-revalidate, max-age=0";
  }

  if (cacheBustedPrefix && filename.startsWith(cacheBustedPrefix)) {
    // js & css files should have a hash so if index.html change: the js & css
    // file will change. It allows to have an aggressive cache for js & css files.
    return "max-age=31536000";
  }

  return undefined;
};